@@ -45,7 +45,7 @@ struct module_state {
  *
  * Returns a new ref */
 static PyObject* _error(char* name) {
-    PyObject* error;
+    PyObject* error = NULL;
     PyObject* errors = PyImport_ImportModule("pymongo.errors");
     if (!errors) {
         return NULL;
@@ -68,8 +68,6 @@ static int buffer_write_bytes_ssize_t(buffer_t buffer, const char* data, Py_ssiz
 
 static PyObject* _cbson_query_message(PyObject* self, PyObject* args) {
     /* NOTE just using a random number as the request_id */
-    struct module_state *state = GETSTATE(self);
-
     int request_id = rand();
     unsigned int flags;
     char* collection_name = NULL;
@@ -77,13 +75,17 @@ static PyObject* _cbson_query_message(PyObject* self, PyObject* args) {
     int begin, cur_size, max_size = 0;
     int num_to_skip;
     int num_to_return;
-    PyObject* query;
-    PyObject* field_selector;
-    PyObject* options_obj;
+    PyObject* query = NULL;
+    PyObject* field_selector = NULL;
+    PyObject* options_obj = NULL;
     codec_options_t options;
     buffer_t buffer = NULL;
     int length_location, message_length;
     PyObject* result = NULL;
+    struct module_state *state = GETSTATE(self);
+    if (!state) {
+        return NULL;
+    }
 
     if (!(PyArg_ParseTuple(args, "Iet#iiOOO",
                           &flags,
@@ -216,17 +218,15 @@ fail:
  * only checked *after* generating the entire message.
  */
 static PyObject* _cbson_op_msg(PyObject* self, PyObject* args) {
-    struct module_state *state = GETSTATE(self);
-
     /* NOTE just using a random number as the request_id */
     int request_id = rand();
     unsigned int flags;
-    PyObject* command;
+    PyObject* command = NULL;
     char* identifier = NULL;
     Py_ssize_t identifier_length = 0;
-    PyObject* docs;
-    PyObject* doc;
-    PyObject* options_obj;
+    PyObject* docs = NULL;
+    PyObject* doc = NULL;
+    PyObject* options_obj = NULL;
     codec_options_t options;
     buffer_t buffer = NULL;
     int length_location, message_length;
@@ -234,6 +234,10 @@ static PyObject* _cbson_op_msg(PyObject* self, PyObject* args) {
     int max_doc_size = 0;
     PyObject* result = NULL;
     PyObject* iterator = NULL;
+    struct module_state *state = GETSTATE(self);
+    if (!state) {
+        return NULL;
+    }
 
     /*flags, command, identifier, docs, opts*/
     if (!(PyArg_ParseTuple(args, "IOet#OO",
@@ -531,15 +535,18 @@ static PyObject*
 _cbson_encode_batched_op_msg(PyObject* self, PyObject* args) {
     unsigned char op;
     unsigned char ack;
-    PyObject* command;
-    PyObject* docs;
+    PyObject* command = NULL;
+    PyObject* docs = NULL;
     PyObject* ctx = NULL;
     PyObject* to_publish = NULL;
     PyObject* result = NULL;
-    PyObject* options_obj;
+    PyObject* options_obj = NULL;
     codec_options_t options;
     buffer_t buffer;
     struct module_state *state = GETSTATE(self);
+    if (!state) {
+        return NULL;
+    }
 
     if (!(PyArg_ParseTuple(args, "bOObOO",
                           &op, &command, &docs, &ack,
@@ -585,15 +592,18 @@ _cbson_batched_op_msg(PyObject* self, PyObject* args) {
     unsigned char ack;
     int request_id;
     int position;
-    PyObject* command;
-    PyObject* docs;
+    PyObject* command = NULL;
+    PyObject* docs = NULL;
     PyObject* ctx = NULL;
     PyObject* to_publish = NULL;
     PyObject* result = NULL;
-    PyObject* options_obj;
+    PyObject* options_obj = NULL;
     codec_options_t options;
     buffer_t buffer;
     struct module_state *state = GETSTATE(self);
+    if (!state) {
+        return NULL;
+    }
 
     if (!(PyArg_ParseTuple(args, "bOObOO",
                           &op, &command, &docs, &ack,
@@ -858,15 +868,18 @@ _cbson_encode_batched_write_command(PyObject* self, PyObject* args) {
     char *ns = NULL;
     unsigned char op;
     Py_ssize_t ns_len;
-    PyObject* command;
-    PyObject* docs;
+    PyObject* command = NULL;
+    PyObject* docs = NULL;
     PyObject* ctx = NULL;
     PyObject* to_publish = NULL;
     PyObject* result = NULL;
-    PyObject* options_obj;
+    PyObject* options_obj = NULL;
     codec_options_t options;
     buffer_t buffer;
     struct module_state *state = GETSTATE(self);
+    if (!state) {
+        return NULL;
+    }
 
     if (!(PyArg_ParseTuple(args, "et#bOOOO", "utf-8",
                           &ns, &ns_len, &op, &command, &docs,
@@ -925,43 +938,41 @@ static PyMethodDef _CMessageMethods[] = {
     {NULL, NULL, 0, NULL}
 };
 
-#define INITERROR return NULL
+#define INITERROR return -1;
 static int _cmessage_traverse(PyObject *m, visitproc visit, void *arg) {
-    Py_VISIT(GETSTATE(m)->_cbson);
-    Py_VISIT(GETSTATE(m)->_max_bson_size_str);
-    Py_VISIT(GETSTATE(m)->_max_message_size_str);
-    Py_VISIT(GETSTATE(m)->_max_split_size_str);
-    Py_VISIT(GETSTATE(m)->_max_write_batch_size_str);
+    struct module_state *state = GETSTATE(m);
+    if (!state) {
+        return 0;
+    }
+    Py_VISIT(state->_cbson);
+    Py_VISIT(state->_max_bson_size_str);
+    Py_VISIT(state->_max_message_size_str);
+    Py_VISIT(state->_max_split_size_str);
+    Py_VISIT(state->_max_write_batch_size_str);
     return 0;
 }
 
 static int _cmessage_clear(PyObject *m) {
-    Py_CLEAR(GETSTATE(m)->_cbson);
-    Py_CLEAR(GETSTATE(m)->_max_bson_size_str);
-    Py_CLEAR(GETSTATE(m)->_max_message_size_str);
-    Py_CLEAR(GETSTATE(m)->_max_split_size_str);
-    Py_CLEAR(GETSTATE(m)->_max_write_batch_size_str);
+    struct module_state *state = GETSTATE(m);
+    if (!state) {
+        return 0;
+    }
+    Py_CLEAR(state->_cbson);
+    Py_CLEAR(state->_max_bson_size_str);
+    Py_CLEAR(state->_max_message_size_str);
+    Py_CLEAR(state->_max_split_size_str);
+    Py_CLEAR(state->_max_write_batch_size_str);
     return 0;
 }
 
-static struct PyModuleDef moduledef = {
-        PyModuleDef_HEAD_INIT,
-        "_cmessage",
-        NULL,
-        sizeof(struct module_state),
-        _CMessageMethods,
-        NULL,
-        _cmessage_traverse,
-        _cmessage_clear,
-        NULL
-};
-
-PyMODINIT_FUNC
-PyInit__cmessage(void)
+/* Multi-phase extension module initialization code.
+ * See https://peps.python.org/pep-0489/.
+*/
+static int
+_cmessage_exec(PyObject *m)
 {
     PyObject *_cbson = NULL;
     PyObject *c_api_object = NULL;
-    PyObject *m = NULL;
     struct module_state* state = NULL;
 
     /* Store a reference to the _cbson module since it's needed to call some
@@ -984,13 +995,10 @@ PyInit__cmessage(void)
         goto fail;
     }
 
-    /* Returns a new reference. */
-    m = PyModule_Create(&moduledef);
-    if (m == NULL) {
+    state = GETSTATE(m);
+    if (state == NULL) {
         goto fail;
     }
-
-    state = GETSTATE(m);
     state->_cbson = _cbson;
     if (!((state->_max_bson_size_str = PyUnicode_FromString("max_bson_size")) &&
         (state->_max_message_size_str = PyUnicode_FromString("max_message_size")) &&
@@ -1000,8 +1008,7 @@ PyInit__cmessage(void)
         }
 
     Py_DECREF(c_api_object);
-
-    return m;
+    return 0;
 
 fail:
     Py_XDECREF(m);
@@ -1009,3 +1016,34 @@ fail:
     Py_XDECREF(_cbson);
     INITERROR;
 }
+
+
+static PyModuleDef_Slot _cmessage_slots[] = {
+    {Py_mod_exec, _cmessage_exec},
+#ifdef Py_MOD_MULTIPLE_INTERPRETERS_SUPPORTED
+    {Py_mod_multiple_interpreters, Py_MOD_MULTIPLE_INTERPRETERS_SUPPORTED},
+#endif
+#if PY_VERSION_HEX >= 0x030D0000
+    {Py_mod_gil, Py_MOD_GIL_NOT_USED},
+#endif
+    {0, NULL},
+};
+
+
+static struct PyModuleDef moduledef = {
+        PyModuleDef_HEAD_INIT,
+        "_cmessage",
+        NULL,
+        sizeof(struct module_state),
+        _CMessageMethods,
+        _cmessage_slots,
+        _cmessage_traverse,
+        _cmessage_clear,
+        NULL
+};
+
+PyMODINIT_FUNC
+PyInit__cmessage(void)
+{
+    return PyModuleDef_Init(&moduledef);
+}
